/*******************************************************
This file orignally by Alvin Leung, modified by Stephanie
Weirich for inclusion in the program _Shade_.
********************************************************/
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>


/*********************************************************
Both of these functions do the exact same thing, except 
that the first reads binary data, while the second ascii.
********************************************************/

#ifdef ANSI
void Read_binary_polygon_data(char *DataFileName,
			      int *Num_Vertices, double ***Vertices,
			      int *Num_Polygons, int ***Polygons)
#else
     void Read_binary_polygon_data(DataFileName, Num_Vertices, Vertices,
				   Num_Polygons, Polygons)
     char *DataFileName;
     int *Num_Vertices;
     double ***Vertices;
     int *Num_Polygons;
     int ***Polygons;
#endif

{
  char  *Subroutine_name = "Read_binary_polygon_data()";
  int    DataFile;
  int    Buffer_Size;               
  char   *Buffer;
  double *double_Pointer;
  int    *int_Pointer;
  int    i;
  int    Index;

   if ((DataFile = open(DataFileName, O_RDONLY)) == -1)
   {
      printf("opening file %s\n", DataFileName);
      perror(Subroutine_name);
      exit(1);
   }
   read(DataFile, Num_Vertices, sizeof(int));

#ifdef DEBUG_Read_polygon_data
   printf("Number_Vertices = %d\n", *Num_Vertices);
#endif

   if ((*Vertices = (double **) malloc(*Num_Vertices * sizeof(double *)))
       == NULL)   {
      perror(Subroutine_name);
      exit(1);
   }
   Buffer_Size = *Num_Vertices * 3 * sizeof(double);
   if ((Buffer = (char *) malloc(Buffer_Size)) == NULL)   {
      perror(Subroutine_name);
      exit(1);
   }
   double_Pointer = (double *) Buffer;
   for (i = 0, Index = 0; i < *Num_Vertices; i++, Index += 3)
      (*Vertices)[i] = &(double_Pointer[Index]);
   read(DataFile, Buffer, Buffer_Size);
#ifdef DEBUG_Read_polygon_data
   for (i = 0; i < *Num_Vertices; i++)
      printf("%lf %lf %lf\n", (*Vertices)[i][0], (*Vertices)[i][1],
             (*Vertices)[i][2]);
#endif
   read(DataFile, Num_Polygons, sizeof(int));
#ifdef DEBUG_Read_polygon_data
   printf("Number_Polygons = %d\n", *Num_Polygons);
#endif
   if (((*Polygons) = (int **) malloc(*Num_Polygons * sizeof(int *))) == NULL)
   {
      perror(Subroutine_name);
      exit(1);
   }
   Buffer_Size = *Num_Polygons * 3 * sizeof(int);
   if ((Buffer = (char *) malloc(Buffer_Size)) == NULL)
   {
      perror(Subroutine_name);
      exit(1);
   }
   int_Pointer = (int *) Buffer;
   for (i = 0, Index = 0; i < *Num_Polygons; i++, Index += 3)
      (*Polygons)[i] = &(int_Pointer[Index]);
   read(DataFile, Buffer, Buffer_Size);
#ifdef DEBUG_Read_polygon_data
   for (i = 0; i < *Num_Polygons; i++)
      printf("%d %d %d\n", (*Polygons)[i][0], (*Polygons)[i][1],
             (*Polygons)[i][2]);
#endif
   close(DataFile);
}


#ifdef ANSI
void Read_ascii_polygon_data(char *DataFileName,
			      int *Num_Vertices, double ***Vertices,
			      int *Num_Polygons, int ***Polygons)
#else
     void Read_ascii_polygon_data(DataFileName, Num_Vertices, Vertices,
				  Num_Polygons, Polygons)
     char *DataFileName;
     int *Num_Vertices;
     double ***Vertices;
     int *Num_Polygons;
     int ***Polygons;
#endif
{ 
  char  *Subroutine_name = "Read_ascii_polygon_data()";
  FILE*  DataFile;
  int    Buffer_Size;               
  char   *Buffer;
  double *double_Pointer;
  int    *int_Pointer;
  int    i;
  int    Index;
  
  if ((DataFile = fopen(DataFileName,"r")) == NULL)
   {
     printf("opening file %s\n", DataFileName);
     perror(Subroutine_name);
     exit(1);
   }
  fscanf(DataFile, "%d", Num_Vertices);

#ifdef DEBUG_Read_polygon_data
   printf("Number_Vertices = %d\n", *Num_Vertices);
#endif
  
  if ((*Vertices = (double **) malloc(*Num_Vertices * sizeof(double *)))
      == NULL)   {
    perror(Subroutine_name);
    exit(1);
  }
  for (i=0; i< *Num_Vertices; i++)
    {
      if ( ((*Vertices)[i]=(double *)malloc(3*sizeof(double))) == NULL){
	perror(Subroutine_name);
	exit(1);
      }
      fscanf(DataFile,"%lf %lf %lf", &(*Vertices)[i][0], &(*Vertices)[i][1],
	     &(*Vertices)[i][2]);
    }
  
#ifdef DEBUG_Read_polygon_data
   for (i = 0; i < *Num_Vertices; i++)
      printf("%lf %lf %lf\n", (*Vertices)[i][0], (*Vertices)[i][1],
             (*Vertices)[i][2]);
#endif      
 
fscanf(DataFile, "%d", Num_Polygons);
#ifdef DEBUG_Read_polygon_data
   printf("Number_Polygons = %d\n", *Num_Polygons);
#endif
   if (((*Polygons) = (int **) malloc(*Num_Polygons * sizeof(int *))) == NULL)
   {
      perror(Subroutine_name);
      exit(1);
   }
  for (i=0; i< *Num_Polygons; i++) 
    {
      if ( ((*Polygons)[i]=(int *)malloc(3*sizeof(int))) == NULL){
	perror(Subroutine_name);
	exit(1);
      }
      fscanf(DataFile,"%d %d %d", &(*Polygons)[i][0], &(*Polygons)[i][1],
	     &(*Polygons)[i][2]);
    }


#ifdef DEBUG_Read_polygon_data
   for (i = 0; i < *Num_Polygons; i++)
      printf("%d %d %d\n", (*Polygons)[i][0], (*Polygons)[i][1],
             (*Polygons)[i][2]);
#endif
   close(DataFile);
}

 
